@@ -37,15 +37,12 @@ class xAIConfig(BaseConfig):
     def _setModels(self):
         self.RModel = os.getenv("GROK_RESPONSE_MODEL", "grok-4")
         self.VModel = os.getenv("GROK_VISION_MODEL", "grok-4")
+        self.GModel = os.getenv("GROK_GENERATION_MODEL", "grok-2-image")
 
     # ---------------------------------------------------------
     # Response
     # ---------------------------------------------------------
     def Response(self, **kwargs) -> str:
-        # keys = [
-        #     "model", "system", "user", "skills", "tools", "choice",
-        #     "show", "effort", "budget", "tokens", "files", "collect", "verbose"
-        # ]
         keys = self.getKeys("response")
         params = self.extractKwargs(kwargs, keys)
 
@@ -60,7 +57,6 @@ class xAIConfig(BaseConfig):
 
         if params["tools"]:
             args["tools"] = params["tools"]
-            #args["tool_choice"] = params["choice"]
             args["tool_choice"] = self._mapChoice(params.get("choice"))
 
         if params["skills"]:
@@ -70,17 +66,7 @@ class xAIConfig(BaseConfig):
             if additionalInfo:
                 messages.append(formatJsonInput("user", additionalInfo))
 
-        # intent = self.extractIntent(params["user"])
-        # fileInfo = extractFileInfo(intent)
-        # if fileInfo:
-        #     messages.append(formatJsonInput("user", fileInfo))
-
-        # files = params["files"]
-        # if files:
-        #     fileInfo = str(extractText(files))
-        #     if fileInfo:
-        #         messages.append(formatJsonInput("user", fileInfo))
-        messages += self._DocFiles(params)
+        messages += self._docFiles(params)
 
         if supportsReasoning(params["model"]):
             print("Using reasoning")
@@ -92,17 +78,13 @@ class xAIConfig(BaseConfig):
             if params["budget"]:
                 args["max_tokens"] = params["budget"]
 
-        response = self._endPoint(**args)# self.client.chat.completions.create(**args)
+        response = self._endPoint(**args)
         return response if params["verbose"] else response.choices[0].message.content
 
     # ---------------------------------------------------------
     # Vision
     # ---------------------------------------------------------
     def Vision(self, **kwargs):
-        # keys = [
-        #     "model", "system", "user", "skills", "tools", "choice",
-        #     "show", "effort", "budget", "tokens", "files", "collect", "verbose"
-        # ]
         keys = self.getKeys("vision")
         params = self.extractKwargs(kwargs, keys)
 
@@ -121,17 +103,39 @@ class xAIConfig(BaseConfig):
 
         if params["creativity"]:
             args["temperature"] = params["creativity"]
-        
-        response = self._endPoint(**args)# self.client.chat.completions.create(**args)
+
+        response = self._endPoint(**args)
         return response if params["verbose"] else response.choices[0].message.content
 
+    # ---------------------------------------------------------
+    # Generation
+    # ---------------------------------------------------------
+    def Generation(self, **kwargs):
+        keys = self.getKeys("generation")
+        params = self.extractKwargs(kwargs, keys)
+
+        model = params.get("model")
+        user = params["user"]
+        output = params.get("output")
+
+        try:
+            response = self.client.images.generate(model=model, prompt=user)
+        except Exception:
+            response = self.client.images.generate(model=self.GModel, prompt=user)
+
+        img = self._normalizeImage(url=response.data[0].url)
+        return self._returnImage(img, output)
+    
+    # ---------------------------------------------------------
+    # Skills
+    # ---------------------------------------------------------
     def processSkills(self, instructions, user, tokens) -> str:
         messages = []
         messages.append(formatJsonInput("system", instructions))
         intent = self.extractIntent(user)
         messages.append(formatJsonInput("user", intent))
         args = self._getArgs(self.RModel, messages, tokens)
-        response = self._endPoint(**args)# self.client.chat.completions.create(**args)
+        response = self._endPoint(**args)
         return response.choices[0].message.content
 
     def _endPoint(self, **args) -> str:
@@ -153,7 +157,6 @@ class xAIConfig(BaseConfig):
         }
         return mapping[(choice or "auto").lower()]
 
-
     def _mediaFiles(self, files, collect):
         images = []
         for path in files:
@@ -165,12 +168,12 @@ class xAIConfig(BaseConfig):
             })
         return images
 
-    def _DocFiles(self, params):
+    def _docFiles(self, params):
         messages = []
-        intent = self.extractIntent(params["user"])
-        fileInfo = extractFileInfo(intent)
-        if fileInfo:
-            messages.append(formatJsonInput("user", fileInfo))
+        # intent = self.extractIntent(params["user"])
+        # fileInfo = extractFileInfo(intent)
+        # if fileInfo:
+        #     messages.append(formatJsonInput("user", fileInfo))
         files = params["files"]
         if files:
             fileInfo = str(extractText(files))
