@@ -1,4 +1,4 @@
-import base64
+﻿import base64
 import collections.abc
 import io
 import os
@@ -47,9 +47,11 @@ if CONTRIBUTORS:
     HOLOAI_MSG += f" Contributors: {addNames(CONTRIBUTORS)}."
 
 ABOUT = (
-    "HoloAI is a modular, provider-agnostic AI framework designed for rapid prototyping and production workloads.\n"
-    f"It supports seamless integration with {addNames(PROVIDERS)} with more coming soon, and includes utilities for structured prompts,\n"
-    "vision workflows, and safe deployment. Created for extensibility and clarity."
+    "HoloAI is a modular, provider-agnostic AI framework built for both rapid prototyping and "
+    "production-grade workloads.\n"
+    f"It offers seamless integration with {addNames(PROVIDERS)}, with additional providers on the way.\n"
+    "Designed for clarity, extensibility, and safety, HoloAI adapts to diverse workflows while making "
+    "deployment straightforward and reliable."
 )
 
 LEGAL_NOTICE = (
@@ -133,8 +135,8 @@ def getContextLength(model_path):
         return 512
 
 def parseInstructions(kwargs):
-    system = kwargs.get("system")
-    instructions = kwargs.get("instructions")
+    system = kwargs.pop("system", None)
+    instructions = kwargs.pop("instructions", None)
     if system and instructions:
         sections = {
             "Main": system,
@@ -161,31 +163,56 @@ def validateResponseArgs(model, user):
     if not user:
         raise ValueError("User input cannot be None or empty.")
 
-def validateVisionArgs(model, user, paths):
+def validateVisionArgs(model, user, files):
     validateResponseArgs(model, user)
-    if not paths or not isinstance(paths, list) or len(paths) == 0:
+    if not files or not isinstance(files, list) or len(files) == 0:
         raise ValueError("paths must be a list with at least one item.")
 
+def validateGenerationArgs(model, user):
+    validateResponseArgs(model, user)
+
+# def parseModels(models):
+#     if models is None:
+#         raise ValueError("You must specify at least one model (string, list/tuple, or dict).")
+#     if isinstance(models, str):
+#         return {'response': models, 'vision': models}
+#     if isinstance(models, (list, tuple)):
+#         if not models:
+#             raise ValueError("Model list/tuple must have at least one value.")
+#         response = models[0]
+#         vision = models[1] if len(models) > 1 else response
+#         return {'response': response, 'vision': vision}
+#     if isinstance(models, dict):
+#         models = {k.lower(): v for k, v in models.items()}
+#         response = models.get('response') or models.get('vision')
+#         vision = models.get('vision') or response
+#         if not response or not vision:
+#             raise ValueError("Dict must contain at least 'response' or 'vision' key.")
+#         return {'response': response, 'vision': vision}
+#     raise TypeError("models must be a string, list/tuple, or dict.")
 def parseModels(models):
     if models is None:
         raise ValueError("You must specify at least one model (string, list/tuple, or dict).")
     if isinstance(models, str):
-        return {'response': models, 'vision': models}
+        return {'response': models, 'vision': models, 'generation': models}
     if isinstance(models, (list, tuple)):
         if not models:
             raise ValueError("Model list/tuple must have at least one value.")
         response = models[0]
         vision = models[1] if len(models) > 1 else response
-        return {'response': response, 'vision': vision}
+        generation = models[2] if len(models) > 2 else response
+        return {'response': response, 'vision': vision, 'generation': generation}
     if isinstance(models, dict):
         models = {k.lower(): v for k, v in models.items()}
-        response = models.get('response') or models.get('vision')
+        response = models.get('response')
+        if not response:
+            raise ValueError("Dict must contain at least a 'response' key.")
         vision = models.get('vision') or response
-        if not response or not vision:
-            raise ValueError("Dict must contain at least 'response' or 'vision' key.")
-        return {'response': response, 'vision': vision}
+        generation = models.get('generation') or response
+        return {'response': response, 'vision': vision, 'generation': generation}
     raise TypeError("models must be a string, list/tuple, or dict.")
 
+
 def isStructured(obj):
     return (
         isinstance(obj, list)
