@@ -1,51 +1,7 @@
 ﻿
-# import logging
-# import os
-# import re
-# import threading
-# from datetime import datetime
-# from dotenv import load_dotenv
-
-# from .HAIUtils.HAIUtils import (
-#     getFrameworkInfo,
-#     formatJsonInput,
-#     formatJsonExtended,
-#     parseJsonInput,
-#     formatTypedInput,
-#     formatTypedExtended,
-#     parseTypedInput,
-#     parseInstructions,
-#     parseModels,
-#     isStructured,
-#     safetySettings,
-#     extractMediaInfo,
-#     getFrames
-# )
-
-# load_dotenv()
-# logger = logging.getLogger(__name__)
-
-# def isKeySet(envKey):
-#     return os.getenv(envKey) is not None
-
-# PROVIDERS = {
-#     "OPENAI_API_KEY":    (".HAIConfigs.OpenAIConfig",   "OpenAIConfig",   "openai"),
-#     "ANTHROPIC_API_KEY": (".HAIConfigs.AnthropicConfig","AnthropicConfig","anthropic"),
-#     "GOOGLE_API_KEY":    (".HAIConfigs.GoogleConfig",   "GoogleConfig",   "google"),
-#     "GROQ_API_KEY":      (".HAIConfigs.GroqConfig",     "GroqConfig",     "groq"),
-# }
-
-# providerMap = {}
-
-# for key, (module, clsName, mapKey) in PROVIDERS.items():
-#     if isKeySet(key):
-#         try:
-#             mod = __import__(module, fromlist=[clsName])
-#             providerMap[mapKey] = getattr(mod, clsName)()
-#         except ImportError as e:
-#             # Optionally log or warn here
-#             pass
-
+# Pip Package: HoloAI
+import asyncio
+import inspect
 import logging
 import os
 import re
@@ -53,6 +9,8 @@ import threading
 from importlib import import_module
 from datetime import datetime
 from dotenv import load_dotenv
+from typing import Iterable, Union, List, Dict, Any
+import tempfile, mimetypes, urllib.request
 
 from .HAIUtils.HAIUtils import (
     getFrameworkInfo,
@@ -66,7 +24,6 @@ from .HAIUtils.HAIUtils import (
     parseModels,
     isStructured,
     safetySettings,
-    extractMediaInfo,
     getFrames
 )
 
@@ -127,7 +84,7 @@ def setProvider(apiInput=None):
     return providerMap
 
 MODELS = {
-    ("gpt", "o"): "openai",
+    ("gpt", "o", "dall-e-3"): "openai",
     ("claude",): "anthropic",
     ("llama", "meta-llama", "gemma2", "qwen", "deepseek",): "groq",
     ("gemini", "gemma",): "google",
@@ -201,6 +158,8 @@ class HoloAI:
                 - list/tuple: [response_model, vision_model].
                 - dict: {'response': ..., 'vision': ...}.
             - system/instructions: (str) System prompt or additional instructions (Optional).
+                system = "You are a helpful assistant."
+                instructions = "You can answer questions and provide information."
             - user/input: (str or list) The main user input (Required).
                 Accepts a single prompt string or a message history (list of messages).
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
@@ -212,7 +171,7 @@ class HoloAI:
                       actions=actions
                 If only actions is provided (without skills), this is not allowed and will result in an error.
             - tools: (list) Tools to use (Optional) [tools].
-            - toolCoice/tool_choice: (str) Controls model tool-calling behavior.
+            - choice/tool_choice: (str) Controls model tool-calling behavior.
                 Valid values:
                     - 'auto': Model decides when to call tools (default)
                     - 'required': Model must call a tool (forces function/tool call)
@@ -238,6 +197,8 @@ class HoloAI:
                 - list/tuple: [response_model, vision_model].
                 - dict: {'response': ..., 'vision': ...}.
             - system/instructions: (str) System prompt or additional instructions (Optional).
+                system = "You are a helpful assistant."
+                instructions = "You can answer questions and provide information."
             - user/input: (str or list) The main user input (Required).
                 Accepts a single prompt string or a message history (list of messages).
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
@@ -249,7 +210,7 @@ class HoloAI:
                       actions=actions
                 If only actions is provided (without skills), this is not allowed and will result in an error.
             - tools: (list) Tools to use (Optional) [tools].
-            - toolCoice/tool_choice: (str) Controls model tool-calling behavior.
+            - choice/tool_choice: (str) Controls model tool-calling behavior.
                 Valid values:
                     - 'auto': Model decides when to call tools (default)
                     - 'required': Model must call a tool (forces function/tool call)
@@ -270,11 +231,13 @@ class HoloAI:
         :param kwargs: Keyword arguments to customize the request.
             - model: (str) The model to use (Required).
             - system/instructions: (str) System prompt or additional instructions (Optional).
+                system = "You are a helpful assistant."
+                instructions = "You can answer questions and provide information."
             - user/input: (str or list) The main user input (Required). 
                 Accepts a single prompt string or a message history (list of messages). 
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
             - tools: (list) Tools to use (Optional) [tools].
-            - toolCoice/tool_choice: (str) Controls model tool-calling behavior.
+            - choice/tool_choice: (str) Controls model tool-calling behavior.
                 Valid values:
                     - 'auto': Model decides when to call tools (default)
                     - 'required': Model must call a tool (forces function/tool call)
@@ -295,11 +258,13 @@ class HoloAI:
         :param kwargs: Keyword arguments to customize the request.
             - model: (str) The model to use (Required).
             - system/instructions: (str) System prompt or additional instructions (Optional).
+                system = "You are a helpful assistant."
+                instructions = "You can answer questions and provide information."
             - user/input: (str or list) The main user input (Required). 
                 Accepts a single prompt string or a message history (list of messages). 
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
             - tools: (list) Tools to use (Optional) [tools].
-            - toolCoice/tool_choice: (str) Controls model tool-calling behavior.
+            - choice/tool_choice: (str) Controls model tool-calling behavior.
                 Valid values:
                     - 'auto': Model decides when to call tools (default)
                     - 'required': Model must call a tool (forces function/tool call)
@@ -318,6 +283,8 @@ class HoloAI:
         :param kwargs: Keyword arguments to customize the request.
             - model: (str) The model to use (Required).
             - system/instructions: (str) System prompt or additional instructions (Optional).
+                system = "You are a helpful assistant."
+                instructions = "You can answer questions and provide information."
             - user/input: (str or list) The main user input (Required). 
                 Accepts a single prompt string or a message history (list of messages). 
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
@@ -330,18 +297,39 @@ class HoloAI:
         """
         return self._routeVision(**kwargs)
 
+    def Generation(self, **kwargs):
+        """
+        Get a Generation response from the Generation model.
+        :param kwargs: Keyword arguments to customize the request.
+            - model: (str) The model to use (Required).
+            - system/instructions: (str) System prompt or additional instructions (Optional).
+                system = "You are a helpful assistant."
+                instructions = "You can answer questions and provide information."
+            - user/input: (str or list) The main user input (Required). 
+                Accepts a single prompt string or a message history (list of messages). 
+                Both 'user' and 'input' are interchangeable; use either (Preferred: input).
+            - output: (str) Path to a file to use (Optional).
+                If a path is provided, it will be used to save the generated image for you.
+        Returns:
+            PIL.Image.Image: The generated image. If a `output` is provided, the image
+            is also saved to that location before being returned.
+        """
+        return self._routeGeneration(**kwargs)
+
     def Agent(self, **kwargs):
         """
         Get a Response from the Agent model.
         :param kwargs: Keyword arguments to customize the request.
-            - task: (str) Task type ('response', 'reasoning', 'vision') (Optional (default: 'response')).
+            - task: (str) Task type ('response', 'reasoning', 'vision', 'generation') (Optional (default: 'response')).
             - model: (str) The model to use (Required).
             - system/instructions: (str) System prompt or additional instructions (Optional).
+                system = "You are a helpful assistant."
+                instructions = "You can answer questions and provide information."
             - user/input: (str or list) The main user input (Required). 
                 Accepts a single prompt string or a message history (list of messages). 
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
             - tools: (list) Tools to use (Optional) [tools].
-            - toolCoice/tool_choice: (str) Controls model tool-calling behavior.
+            - choice/tool_choice: (str) Controls model tool-calling behavior.
                 Valid values:
                     - 'auto': Model decides when to call tools (default)
                     - 'required': Model must call a tool (forces function/tool call)
@@ -353,13 +341,19 @@ class HoloAI:
             - files: (list) List of file paths can be past in manually or during runtime (default: empty list).
             - collect: (int) Number of frames to collect (default: 10).
             - verbose: (bool) Return verbose output (Optional (default: False)).
-        :return: A Reasoning, Response, or Vision object.
+        :return:
+            - If 'task' is 'reasoning' or 'response', returns a Response object.
+            - If 'task' is 'vision', returns a Vision object.
+            - If 'task' is 'generation', returns a Generation object.
+                PIL.Image.Image: The generated image. If a `output` is provided, the image
+                is also saved to that location before being returned.
         """
         task = kwargs.get('task', 'response').lower()
         taskMap = {
-            'reasoning': self._routeResponse,
-            'response': self._routeResponse,
-            'vision': self._routeVision,
+            'reasoning':  self._routeResponse,
+            'response':   self._routeResponse,
+            'vision':     self._routeVision,
+            'generation': self._routeGeneration,
         }
         if task not in taskMap:
             raise ValueError(f"Unknown task: '{task}'. Supported tasks: {list(taskMap.keys())}")
@@ -368,50 +362,258 @@ class HoloAI:
     #------------- Utility Methods -------------#
     def _routeCompletion(self, **kwargs):
         kwargs  = {k.lower(): v for k, v in kwargs.items()}
-        models  = kwargs.get("models") or kwargs.get("model")
-        raw     = kwargs.get("input") or kwargs.get("user")
-        system  = parseInstructions(kwargs)
-        verbose = kwargs.get("verbose", False)
-        if models is None or raw is None:
-            raise ValueError("HoloCompletion requires 'model' or 'models' and input/user")
+        beta    = kwargs.pop("beta", False)
+        models  = kwargs.pop("model", None) or kwargs.pop("models", None)
+        system  = parseInstructions(kwargs)  # popped in parseInstructions
+        raw     = kwargs.pop("input", None) or kwargs.pop("user", None)
+        files   = kwargs.pop("files", None)
 
         models = parseModels(models)
 
-        # 1. Normalize user input
         if isinstance(raw, list):
             last = raw[-1]
-            text = last["content"] if isinstance(last, dict) and "content" in last else str(last)
+            if isinstance(last, dict) and "content" in last:
+                text = str(last["content"])
+            else:
+                text = str(last)
         else:
             text = str(raw)
 
-        # 2. Detect mode
-        images = extractMediaInfo(text)
+        # Detect
+        passedImages = self._extractMediaInfo(files)
+        inlineImages = self._extractMediaInfo(text)
+        passedFiles  = self._extractFileInfo(files)
+        inlineFiles  = self._extractFileInfo(text)
 
-        # 3. Select mode
-        def visionMode():
-            img = images[-1]
-            promptOnly = re.split(re.escape(img), text)[0].strip()
-            return self.Vision(
-                model=models['vision'],
-                system=system,
-                user=promptOnly,
-                files=images,
-                collect=5,
-                verbose=verbose
+        # Normalize paths
+        passedImages = self._normalizePaths(passedImages)
+        inlineImages = self._normalizePaths(inlineImages)
+        passedFiles  = self._normalizePaths(passedFiles)
+        inlineFiles  = self._normalizePaths(inlineFiles)
+        
+        allImages = list(dict.fromkeys([*passedImages, *inlineImages]))
+        allFiles  = list(dict.fromkeys([*passedFiles, *inlineFiles]))
+
+        # Build prompt without any paths
+        cleanedInput = self._cleanInput(text)
+        #print(f"\nCleaned input: {cleanedInput}\n")
+
+        def holoMode():
+            model  = models['response']
+            config = self._getProviderConfig(model)
+            return config.getResponse(
+                model  = model,
+                system = system,
+                input  = raw,
+                **kwargs,
             )
 
         def responseMode():
-            kwargs['model'] = models['response']
-            return self.Response(**kwargs)
+            model  = models['response']
+            config = self._getProviderConfig(model)
+            # skills  = kwargs.pop('skills', None)
+            # actions = kwargs.pop('actions', None)
+            skills, actions = self._popSkillsActions(kwargs)
+            localFiles = [self._downloadUrlToTemp(p) if p.startswith(("http://", "https://")) else p for p in allFiles]
+            # Decide singular vs plural replacement
+            if len(localFiles) == 1:
+                responseInput = re.sub(r"\bthese files\b", "this file", cleanedInput, flags=re.IGNORECASE)
+            else:
+                responseInput = cleanedInput
+            #print(f"Response input: {responseInput}\n")
+            return config.getResponse(
+                model  = model,
+                system = system,
+                input  = responseInput,
+                files  = localFiles, # allFiles,
+                **kwargs,
+            )
+
+        def visionMode():
+            model  = models['vision']
+            config = self._getProviderConfig(model)
+            # skills  = kwargs.pop('skills', None)
+            # actions = kwargs.pop('actions', None)
+            skills, actions = self._popSkillsActions(kwargs)
+            localFiles = [self._downloadUrlToTemp(p) if p.startswith(("http://", "https://")) else p for p in allImages]
+            # Decide singular vs plural replacement
+            if len(localFiles) == 1:
+                visionInput = re.sub(r"\bthese files\b", "this image", cleanedInput, flags=re.IGNORECASE)
+            else:
+                visionInput = re.sub(r"\bfiles\b", "images", cleanedInput)
+            #print(f"Vision input: {visionInput}\n")
+            return config.getVision(
+                model   = model,
+                system  = system,
+                input   = visionInput,
+                files   = localFiles, # allImages,
+                **kwargs,
+            )
+
+        # FUTURE: Uncomment when generation mode is supported DO NOT DELETE
+        def generationMode():
+            # skills  = kwargs.pop('skills', None)
+            # actions = kwargs.pop('actions', None)
+            kwargs.pop('model', None)  # remove model from kwargs
+            kwargs.pop('user', None) or kwargs.pop('input', None)   # remove models from kwargs
+            skills, actions = self._popSkillsActions(kwargs)
+            model  = models['generation']
+            config = self._getProviderConfig(model)
+            user = cleanedInput
+            return config.getGeneration(model=model, user=user, **kwargs)
+
+        async def hybridMode():
+            # skills  = kwargs.pop('skills', None)
+            # actions = kwargs.pop('actions', None)
+            model  = models['response']
+            config = self._getProviderConfig(model)
+            skills, actions = self._popSkillsActions(kwargs)
+            # Run vision and response concurrently in threads
+            visionTask   = asyncio.to_thread(visionMode)
+            responseTask = asyncio.to_thread(responseMode)
+            vision, response = await asyncio.gather(visionTask, responseTask)
+
+            additional = f"[Media Files]:\n{vision}\n[Doc Files]:\n{response}"
+
+            #print(f"Hybrid text: {text}\n")
+            msgs       = f"{cleanedInput}[Media Files]:\n[Doc Files]:\n"
+
+            return config.getResponse(
+                model        = model,
+                system       = system,
+                instructions = additional,
+                input        = msgs,
+                **kwargs,
+            )
+
+        # FUTURE: Uncomment when generation mode is supported DO NOT DELETE
+        if beta:
+            intent = self._routeIntent(models['response'], cleanedInput)
+            if intent == "generation":
+                return generationMode()
 
         modeMap = {
+            "holo": holoMode,
             "vision": visionMode,
             "response": responseMode,
-            # Add more modes here, e.g., "audio": audioMode, etc.
+            "hybrid": hybridMode,
         }
 
-        mode = "vision" if images else "response"
-        return modeMap[mode]()
+        mode = (
+            "hybrid"   if (allImages and allFiles) else
+            "vision"   if allImages                else
+            "response" if allFiles                 else
+            "holo"
+        )
+        #print(f"Using mode: {mode}\n")
+
+        modeFunc = modeMap[mode]
+
+        # Handle both sync and async seamlessly
+        if inspect.iscoroutinefunction(modeFunc):
+            try:
+                loop = asyncio.get_running_loop()
+            except RuntimeError:
+                loop = None
+
+            if loop and loop.is_running():
+                # Already in an event loop -> return coroutine for caller to await
+                #print("[Notice] Running in an existing event loop, returning coroutine.")
+                return modeFunc()
+            else:
+                # No event loop running -> safe to use asyncio.run
+                #print("[Notice] No event loop running, using asyncio.run.")
+                return asyncio.run(modeFunc())
+
+        return modeFunc()
+
+    def _popSkillsActions(self, kwargs):
+        return kwargs.pop("skills", None), kwargs.pop("actions", None)
+
+    def _normalizePaths(self, paths):
+        out = []
+        for p in paths:
+            if isinstance(p, str) and p.startswith(("http://", "https://")):
+                out.append(p)  # leave URL untouched
+            else:
+                out.append(os.path.normpath(p))
+        return out
+
+    def _extractPaths(self, text: Union[str, Iterable[str]], includePatterns: dict, excludePatterns: dict):
+        texts = text if isinstance(text, (list, tuple, set)) else [text]
+
+        # Join all include/exclude patterns into one regex each
+        includeRegex = re.compile("|".join(includePatterns.values()), re.IGNORECASE)
+        excludeRegex = re.compile("|".join(fr"^{p}$" for p in excludePatterns.values()), re.IGNORECASE)
+
+        results = []
+        for t in texts:
+            if not t:
+                continue
+            matches = includeRegex.findall(t)
+            # findall with capturing groups returns tuples; flatten and filter
+            results.extend(
+                p for pair in matches for p in (pair if isinstance(pair, tuple) else (pair,))
+                if p and not excludeRegex.fullmatch(p)
+            )
+        return results
+
+    def _extractMediaInfo(self, text: Union[str, Iterable[str]]):
+        return self._extractPaths(text, self._mediaPatterns(), self._filePatterns())
+
+    def _extractFileInfo(self, text: Union[str, Iterable[str]]):
+        return self._extractPaths(text, self._filePatterns(), self._mediaPatterns())
+
+    def _downloadUrlToTemp(self, url: str) -> str:
+        try:
+            # Guess extension from URL or MIME
+            _, ext = os.path.splitext(url)
+            if not ext:
+                mime, _ = mimetypes.guess_type(url)
+                ext = mimetypes.guess_extension(mime) or ".bin"
+
+            with tempfile.NamedTemporaryFile(delete=False, suffix=ext) as tmp:
+                urllib.request.urlretrieve(url, tmp.name)
+                return tmp.name
+        except Exception as e:
+            print(f"[Warning] Failed to fetch {url}: {e}")
+            return None
+
+    def _mediaPatterns(self):
+        EXT = r'(?:png|jpe?g|gif|webp|bmp|tiff?)'
+        return {
+            "win":  fr'([A-Za-z]:(?:\\|/)[^,\n]*?\.{EXT})',
+            "unix": fr'(/[^ ,\n]*?/[^,\n]*?\.{EXT})',
+            "url":  fr'(https?://[^\s,;()]+?\.{EXT}(?:\?[^\s,;()]*)?)',
+        }
+
+    def _filePatterns(self):
+        EXT = r'(?:docx?|pdf|txt|odt|rtf|xlsx?|pptx?)'
+        return {
+            "win":  fr'([A-Za-z]:(?:\\|/)[^,\n]*?\.{EXT})',
+            "unix": fr'(/[^ ,\n]*?/[^,\n]*?\.{EXT})',
+            "url":  fr'(https?://[^\s,;()]+?\.{EXT}(?:\?[^\s,;()]*)?)',
+        }
+
+    def _cleanInput(self, text: str) -> str:
+        allExts = r'(?:png|jpe?g|gif|webp|bmp|tiff?|docx?|pdf|txt|odt|rtf|xlsx?|pptx?)'
+
+        # Local paths (allow spaces until extension)
+        win  = rf'[A-Za-z]:(?:\\|/)[^,\n]*?\.{allExts}'
+        unix = rf'/[^ ,\n]*?/[^,\n]*?\.{allExts}'
+
+        # URLs ending with target extensions (optional querystring)
+        url  = rf'https?://[^\s,;()]+?\.{allExts}(?:\?[^\s,;()]*)?'
+
+        pathOrUrl = rf'(?:{win}|{unix}|{url})'
+
+        # Remove matches with optional nearby separators
+        text = re.sub(rf'(?:\s*[,;]?\s*){pathOrUrl}(?=(\s|[,;.)]|$))',
+                      '', text, flags=re.IGNORECASE)
+
+        # Collapse leftover punctuation/whitespace artifacts
+        text = re.sub(r'\s*[,;]\s*', ' ', text)
+        return ' '.join(text.split())
 
     def _routeResponse(self, **kwargs):
         kwargs = {k.lower(): v for k, v in kwargs.items()}
@@ -423,10 +625,23 @@ class HoloAI:
         kwargs = {k.lower(): v for k, v in kwargs.items()}
         model  = kwargs.get('model')
         config = self._getProviderConfig(model)
-        notice = kwargs.get('paths')
-        if notice:
-            print(f"[Notice] 'paths' is being deprecated in future releases, please use 'files' instead.")
-        return config.getVision( **kwargs)
+        return config.getVision(**kwargs)
+
+    def _routeGeneration(self, **kwargs):
+        kwargs = {k.lower(): v for k, v in kwargs.items()}
+        model  = kwargs.get('model')
+        config = self._getProviderConfig(model)
+        return config.getGeneration(**kwargs)
+
+    def _routeIntent(self, model, user):
+        system = (
+            "Your task is to determine the user's intent and respond ONLY with one of the following:"
+            "\n- 'response' for text generation"
+            "\n- 'vision' for vision tasks"
+            "\n- 'generation' for image generation tasks"
+        )
+
+        return self.Response(model=model,system=system,input=user)
 
     def isStructured(self, obj):
         """
@@ -520,10 +735,3 @@ class HoloAI:
         """
         return safetySettings(**kwargs)
 
-    def extractMediaInfo(self, text: str):
-        """
-        Extracts image file paths from a given text.
-        Supports both Windows and Unix-style paths.
-        Returns a list of matched image paths.
-        """
-        return extractMediaInfo(text)
